@@ -1,13 +1,10 @@
 
 ## EM for Beta Mixture Models (BMM) with Nc components
 
-## TODO: revisit convergence criteria; consider changing to monitoring
-## relative n and absolute mean; or revisit eps calculation with
-## parameters mu and n.
-
-EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol, eps=0.1)
+EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=FALSE, Niter.max=500, tol, Neps, eps=c(w=0.005,a=0.005,b=0.005))
 {
     N <- length(x)
+    assert_that(N+Nc >= Ninit)
 
     ## check data for 0 and 1 values which are problematic, but may be
     ## valid, depending on a and b. Moving these to eps or 1-eps
@@ -49,6 +46,12 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
             }
         }
         nInit <- pmax(muInit*(1-muInit)/varInit - 1, 1, na.rm=TRUE)
+        ## place the component which recieved the least weight at the
+        ## data center with roughly the variance of the sample
+        cmin <- which.min(KNN$p)
+        muInit[cmin] <- sum(KNN$p * KNN$center)
+        ## muInit[cmin] <- mean(x) ## could be considered here
+        nInit[cmin] <- pmax(muInit[cmin]*(1-muInit[cmin])/var(x) - 1, 1, na.rm=TRUE)
         ##Nmax <- max(2, max(nInit))
         ## ensure n is positive for each cluster; if this is not the
         ## case, sample uniformly from the range of n we have
@@ -58,6 +61,7 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
         ##    nInit[Nsmall] <- runif(sum(Nsmall), 0.5, Nmax)
         ##nInitR <- 0.5 + rlnorm(Nc, log(nInit), log(5)/1.96)
         mixEst <- rbind(KNN$p, nInit*muInit, nInit*(1-muInit))
+        dlink(mixEst) <- identity_dlink
         rownames(mixEst) <- c("w", "a", "b")
     } else {
         mixEst <- mix_init
@@ -69,26 +73,51 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
         print(mixEst)
     }
 
+    ## mixEst parametrization during fitting
+    mixEstPar <- mixEst
+    mixEstPar[1,] <- logit(mixEst[1,,drop=FALSE])
+    mixEstPar[2,] <- log(mixEst[2,])
+    mixEstPar[3,] <- log(mixEst[3,])
+    rownames(mixEstPar) <-  c("w", "la", "lb")
+
+    ## in case tolerance is not specified, then this criteria is
+    ## ignored
     if(missing(tol)) {
-        ## automatic selection of tolerance, based on targeted
-        ## precision of alpha and beta (crude estimate)
-        da <- digamma(mixEst[2,])
-        db <- digamma(mixEst[3,])
-        dab <- digamma(colSums(mixEst[2:3,,drop=FALSE]))
-        tol <- min(abs((dab - da) * eps) + abs((dab - db) * eps))
-        ##if(tol < 0.05)
-        ##    warning("Tolerance set to very low value. Consider increasing eps, the precision of component estimates.")
+        checkTol <- FALSE
+        tol <- -1
+    } else
+        checkTol <- TRUE
+
+    if(missing(Neps)) {
+        ## in case tolerance has been declared, but Neps not, we flag
+        ## to disable checking of running mean convergence check
+        checkEps <- FALSE
+        Neps <- 5
+    } else
+        checkEps <- TRUE
+
+    ## if nothing is specified, we declare convergence based on a
+    ## running mean of differences in parameter estimates
+    if(!checkTol & !checkEps) {
+        checkEps <- TRUE
     }
 
-    if(verbose) {
-        cat("Admissable change in log-likelihood tol =", tol,"\n")
-    }
+    assert_that(Neps > 1)
+    assert_that(ceiling(Neps) == floor(Neps))
 
-    lliLast <- -Inf
-    iter <- 1
+    ## eps can also be given as a single integer which is interpreted
+    ## as number of digits
+    if(length(eps) == 1) eps <- rep(10^(-eps), 3)
+
+    iter <- 0
     logN <- log(N)
     traceMix <- list()
     traceLli <- c()
+    Dlli <- Inf
+    runMixPar <- array(-Inf, dim=c(Neps,3,Nc), dimnames=list(NULL, rownames(mixEstPar), NULL ))
+    runOrder <- 0:(Neps-1)
+    Npar <- Nc + 2*Nc
+    if(Nc == 1) Npar <- Npar - 1
 
     ## find alpha and beta for a given component in log-space
     bmm_ml <- function(c1,c2) {
@@ -101,7 +130,29 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
         }
     }
 
-    while(iter < maxIter) {
+    bmm_ml_grad  <- function(c1,c2) {
+        function(par) {
+            ab <- exp(par)
+            n <- sum(ab)
+            s <- digamma(n)
+            eq1 <- digamma(ab[1]) - s
+            eq2 <- digamma(ab[2]) - s
+            sqTerm1 <- (eq1 - c1)
+            sqTerm2 <- (eq2 - c2)
+
+            trig_n  <- trigamma(n)
+
+            grad1 <- 2 * sqTerm1 * (trigamma(ab[1]) * ab[1] - trig_n * ab[1] ) -
+                2 * sqTerm2 * (trig_n * ab[1])
+
+            grad2 <- -2 * sqTerm1 * (trig_n * ab[2]) +
+                2 * sqTerm2 * (trigamma(ab[2]) * ab[2] - trig_n * ab[2])
+
+            c(grad1, grad2)
+        }
+    }
+
+    while(iter < Niter.max) {
         ## calculate responsabilities from the likelihood terms;
         ## calculations are done in log-space to avoid numerical
         ## difficulties if some points are far away from some
@@ -114,51 +165,77 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
         lnresp <- apply(lli, 1, log_sum_exp)
         ## the log-likelihood is then given by the sum of lresp norms
         lliCur <- sum(lnresp)
-        lliDelta <- lliCur - lliLast
         ## record current state
         traceMix <- c(traceMix, list(mixEst))
         traceLli <- c(traceLli, lliCur)
-        if(verbose) {
-            cat("Iteration ", iter, ": log-likelihood = ", lliCur, "; delta = ", lliDelta, "\n", sep="")
+        if(iter > 1) {
+            ## Dlli is the slope of the log-likelihood evaulated with
+            ## a second order method
+            Dlli <- (traceLli[iter+1] - traceLli[iter - 1])/2
         }
-        if(iter != 1 & lliDelta < tol) {
+        if(Nc > 1) {
+            smean <- apply(runMixPar[order(runOrder),,,drop=FALSE], c(2,3), function(x) mean(abs(diff(x))))
+            eps.converged <- sum(sweep(smean, 1, eps, "-") < 0)
+        } else {
+            smean <- apply(runMixPar[order(runOrder),-1,,drop=FALSE], c(2,3), function(x) mean(abs(diff(x))))
+            eps.converged <- sum(sweep(smean, 1, eps[-1], "-") < 0)
+        }
+        if(is.na(eps.converged)) eps.converged <- 0
+        if(verbose) {
+            cat("Iteration ", iter, ": log-likelihood = ", lliCur, "; Dlli = ", Dlli, "; converged = ", eps.converged, " / ", Npar, "\n", sep="")
+        }
+        if(checkTol & Dlli < tol) {
             break
         }
+        if(iter >= Neps & checkEps & eps.converged == Npar) {
+            break
+        }
+
         ## ... and the (log) responseability matrix follows from this by
         ## appropiate normalization.
         lresp <- sweep(lli, 1, lnresp, "-")
-        resp <- exp(lresp)
+        ##resp <- exp(lresp)
 
         ## mean probability to be in a specific mixture component -> updates
         ## abmEst first colum
         lzSum <- apply(lresp, 2, log_sum_exp)
-        zSum <- exp(lzSum)
+        ##zSum <- exp(lzSum)
         mixEst[1,] <- exp(lzSum - logN)
 
-        c1 <- colSums(Lx * resp)/zSum
-        c2 <- colSums(LxC * resp)/zSum
+        ##c1 <- colSums(Lx * resp)/zSum
+        ##c2 <- colSums(LxC * resp)/zSum
+
+        resp_zscaled  <- exp(sweep(lresp, 2, lzSum, "-"))
+        c1 <- colSums(Lx * resp_zscaled)
+        c2 <- colSums(LxC * resp_zscaled)
 
         ## now solve for new alpha and beta estimates jointly for each
         ## component
         for(i in 1:Nc) {
             theta <- c(log(mixEst[2:3,i]))
+            ##Lest <- optim(theta, bmm_ml(c1[i], c2[i]), gr=bmm_ml_grad(c1[i], c2[i]), method="BFGS", control=list(maxit=500))
+            ## Default would be Nelder-Mead
             Lest <- optim(theta, bmm_ml(c1[i], c2[i]))
-            if(Lest$convergence != 0) {
+            if(Lest$convergence != 0 & Lest$value > 1E-4) {
                 warning("Warning: Component", i, "in iteration", iter, "had convergence problems!")
             }
-            ## TODO: consider adding the gradient to bmm_ml
             mixEst[2:3,i] <- exp(Lest$par)
         }
 
-        lliLast <- lliCur
+        mixEstPar[1,] <- logit(mixEst[1,,drop=FALSE])
+        ind <- 1 + iter %% Neps
+        runMixPar[ind,,] <- mixEstPar
+        runOrder[ind] <- iter
+
         iter <- iter + 1
     }
-    if(iter == maxIter)
+    if(iter == Niter.max)
         warning("Maximum number of iterations reached.")
 
     mixEst <- mixEst[,order(mixEst[1,], decreasing=TRUE),drop=FALSE]
     colnames(mixEst) <- paste("comp", seq(Nc), sep="")
-    class(mixEst) <- c("EM", "EMbmm", "betaMix")
+    dlink(mixEst) <- identity_dlink
+    class(mixEst) <- c("EM", "EMbmm", "betaMix", "mix")
 
     ## give further details
     attr(mixEst, "df") <- Nc-1 + 2*Nc
@@ -169,8 +246,16 @@ EM_bmm_ab <- function(x, Nc, mix_init, Ninit=50, verbose=TRUE, maxIter=500, tol,
 
     attr(mixEst, "tol") <- tol
     attr(mixEst, "traceLli") <- traceLli
-    attr(mixEst, "traceMix") <- lapply(traceMix, function(x) {class(x) <- "betaMix"; x})
+    attr(mixEst, "traceMix") <- lapply(traceMix, function(x) {class(x) <- c("betaMix", "mix"); x})
+    attr(mixEst, "x") <- x
 
     mixEst
 }
 
+
+#' @export
+print.EMbmm <- function(x, ...) {
+    cat("EM for Beta Mixture Model\nLog-Likelihood = ", logLik(x), "\n\n",sep="")
+    NextMethod()
+}
+
