@@ -48,7 +48,7 @@ print(map)
 plot(map)$mix
 
 ## ------------------------------------------------------------------------
-ess(map)
+ess(map, method="moment")
 ess(map, method="morita")
 
 ## ------------------------------------------------------------------------
